# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2013 Boris Pavlovic (boris@pavlovic.me).
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

from migrate.changeset import UniqueConstraint, ForeignKeyConstraint
from sqlalchemy import Boolean
from sqlalchemy import CheckConstraint
from sqlalchemy import Column
from sqlalchemy.engine import reflection
from sqlalchemy.exc import OperationalError
from sqlalchemy.exc import ProgrammingError
from sqlalchemy.ext.compiler import compiles
from sqlalchemy import func
from sqlalchemy import Index
from sqlalchemy import Integer
from sqlalchemy import MetaData
from sqlalchemy import schema
from sqlalchemy.sql.expression import literal_column
from sqlalchemy.sql.expression import UpdateBase
from sqlalchemy.sql import select
from sqlalchemy import String
from sqlalchemy import Table
from sqlalchemy.types import NullType

from vsm.db.sqlalchemy import api as db
from vsm import exception
from vsm.openstack.common.gettextutils import _
from vsm.openstack.common import log as logging
from vsm.openstack.common import timeutils

LOG = logging.getLogger(__name__)

def get_table(engine, name):
    """Returns an sqlalchemy table dynamically from db.

    Needed because the models don't work for us in migrations
    as models will be far out of sync with the current data.
    """
    metadata = MetaData()
    metadata.bind = engine
    return Table(name, metadata, autoload=True)

class InsertFromSelect(UpdateBase):
    def __init__(self, table, select):
        self.table = table
        self.select = select

@compiles(InsertFromSelect)
def visit_insert_from_select(element, compiler, **kw):
    return "INSERT INTO %s %s" % (
        compiler.process(element.table, asfrom=True),
        compiler.process(element.select))

def _get_not_supported_column(col_name_col_instance, column_name):
    try:
        column = col_name_col_instance[column_name]
    except Exception:
        msg = _("Please specify column %s in col_name_col_instance "
                "param. It is required because column has unsupported "
                "type by sqlite).")
        raise exception.NovaException(msg % column_name)

    if not isinstance(column, Column):
        msg = _("col_name_col_instance param has wrong type of "
                "column instance for column %s It should be instance "
                "of sqlalchemy.Column.")
        raise exception.NovaException(msg % column_name)
    return column

def _get_unique_constraints_in_sqlite(migrate_engine, table_name):
    regexp = "CONSTRAINT (\w+) UNIQUE \(([^\)]+)\)"

    meta = MetaData(bind=migrate_engine)
    table = Table(table_name, meta, autoload=True)

    sql_data = migrate_engine.execute(
        """
            SELECT sql
            FROM
                sqlite_master
            WHERE
                type = 'table' AND
                name = :table_name;
        """,
        table_name=table_name
    ).fetchone()[0]

    uniques = set([
        schema.UniqueConstraint(
            *[getattr(table.c, c.strip(' "'))
              for c in cols.split(",")], name=name
        )
        for name, cols in re.findall(regexp, sql_data)
    ])

    return uniques

def _drop_unique_constraint_in_sqlite(migrate_engine, table_name, uc_name,
                                      **col_name_col_instance):
    insp = reflection.Inspector.from_engine(migrate_engine)
    meta = MetaData(bind=migrate_engine)

    table = Table(table_name, meta, autoload=True)
    columns = []
    for column in table.columns:
        if isinstance(column.type, NullType):
            new_column = _get_not_supported_column(col_name_col_instance,
                                                   column.name)
            columns.append(new_column)
        else:
            columns.append(column.copy())

    uniques = _get_unique_constraints_in_sqlite(migrate_engine, table_name)
    table.constraints.update(uniques)

    constraints = [constraint for constraint in table.constraints
                    if not constraint.name == uc_name and
                    not isinstance(constraint, schema.ForeignKeyConstraint)]

    new_table = Table(table_name + "__tmp__", meta, *(columns + constraints))
    new_table.create()

    indexes = []
    for index in insp.get_indexes(table_name):
        column_names = [new_table.c[c] for c in index['column_names']]
        indexes.append(Index(index["name"],
                             *column_names,
                             unique=index["unique"]))
    f_keys = []
    for fk in insp.get_foreign_keys(table_name):
        refcolumns = [fk['referred_table'] + '.' + col
                      for col in fk['referred_columns']]
        f_keys.append(ForeignKeyConstraint(fk['constrained_columns'],
                      refcolumns, table=new_table, name=fk['name']))

    ins = InsertFromSelect(new_table, table.select())
    migrate_engine.execute(ins)
    table.drop()

    [index.create(migrate_engine) for index in indexes]
    for fkey in f_keys:
        fkey.create()
    new_table.rename(table_name)

def drop_unique_constraint(migrate_engine, table_name, uc_name, *columns,
                           **col_name_col_instance):
    """
    This method drops UC from table and works for mysql, postgresql and sqlite.
    In mysql and postgresql we are able to use "alter table" constuction. In
    sqlite is only one way to drop UC:
        1) Create new table with same columns, indexes and constraints
           (except one that we want to drop).
        2) Copy data from old table to new.
        3) Drop old table.
        4) Rename new table to the name of old table.

    :param migrate_engine: sqlalchemy engine
    :param table_name:     name of table that contains uniq constarint.
    :param uc_name:        name of uniq constraint that will be dropped.
    :param columns:        columns that are in uniq constarint.
    :param col_name_col_instance:   contains pair column_name=column_instance.
                            column_instance is instance of Column. These params
                            are required only for columns that have unsupported
                            types by sqlite. For example BigInteger.
    """
    if migrate_engine.name == "sqlite":
        _drop_unique_constraint_in_sqlite(migrate_engine, table_name, uc_name,
                                          **col_name_col_instance)
    else:
        meta = MetaData()
        meta.bind = migrate_engine
        t = Table(table_name, meta, autoload=True)
        uc = UniqueConstraint(*columns, table=t, name=uc_name)
        uc.drop()

def drop_old_duplicate_entries_from_table(migrate_engine, table_name,
                                          use_soft_delete, *uc_column_names):
    """
    This method is used to drop all old rows that have the same values for
    columns in uc_columns.
    """
    meta = MetaData()
    meta.bind = migrate_engine

    table = Table(table_name, meta, autoload=True)
    columns_for_group_by = [table.c[name] for name in uc_column_names]

    columns_for_select = [func.max(table.c.id)]
    columns_for_select.extend(list(columns_for_group_by))

    duplicated_rows_select = select(columns_for_select,
                                    group_by=columns_for_group_by,
                                    having=func.count(table.c.id) > 1)

    for row in migrate_engine.execute(duplicated_rows_select):
        # NOTE(boris-42): Do not remove row that has the biggest ID.
        delete_condition = table.c.id != row[0]
        for name in uc_column_names:
            delete_condition &= table.c[name] == row[name]

        rows_to_delete_select = select([table.c.id]).where(delete_condition)
        for row in migrate_engine.execute(rows_to_delete_select).fetchall():
            LOG.info(_("Deleted duplicated row with id: %(id)s from table: "
                       "%(table)s") % dict(id=row[0], table=table_name))

        if use_soft_delete:
            delete_statement = table.update().\
                    where(delete_condition).\
                    values({
                        'deleted': literal_column('id'),
                        'updated_at': literal_column('updated_at'),
                        'deleted_at': timeutils.utcnow()
                    })
        else:
            delete_statement = table.delete().where(delete_condition)
        migrate_engine.execute(delete_statement)

def check_shadow_table(migrate_engine, table_name):
    """
    This method checks that table with ``table_name`` and corresponding shadow
    table have same columns.
    """
    meta = MetaData()
    meta.bind = migrate_engine

    table = Table(table_name, meta, autoload=True)
    shadow_table = Table(db._SHADOW_TABLE_PREFIX + table_name, meta,
                         autoload=True)

    columns = dict([(c.name, c) for c in table.columns])
    shadow_columns = dict([(c.name, c) for c in shadow_table.columns])

    for name, column in columns.iteritems():
        if name not in shadow_columns:
            raise exception.NovaException(
                _("Missing column %(table)s.%(column)s in shadow table")
                        % {'column': name, 'table': shadow_table.name})
        shadow_column = shadow_columns[name]

        if not isinstance(shadow_column.type, type(column.type)):
            raise exception.NovaException(
                _("Different types in %(table)s.%(column)s and shadow table: "
                  "%(c_type)s %(shadow_c_type)s")
                        % {'column': name, 'table': table.name,
                           'c_type': column.type,
                           'shadow_c_type': shadow_column.type})

    for name, column in shadow_columns.iteritems():
        if name not in columns:
            raise exception.NovaException(
                _("Extra column %(table)s.%(column)s in shadow table")
                        % {'column': name, 'table': shadow_table.name})
    return True

def create_shadow_table(migrate_engine, table_name=None, table=None,
                        **col_name_col_instance):
    """
    This method create shadow table for table with name ``table_name`` or table
    instance ``table``.
    :param table_name: Autoload table with this name and create shadow table
    :param table: Autoloaded table, so just create corresponding shadow table.
    :param col_name_col_instance:   contains pair column_name=column_instance.
                            column_instance is instance of Column. These params
                            are required only for columns that have unsupported
                            types by sqlite. For example BigInteger.
    """
    meta = MetaData(bind=migrate_engine)

    if table_name is None and table is None:
        raise exception.NovaException(_("Specify `table_name` or `table` "
                                        "param"))
    if not (table_name is None or table is None):
        raise exception.NovaException(_("Specify only one param `table_name` "
                                        "`table`"))

    if table is None:
        table = Table(table_name, meta, autoload=True)

    columns = []
    for column in table.columns:
        if isinstance(column.type, NullType):
            new_column = _get_not_supported_column(col_name_col_instance,
                                                   column.name)
            columns.append(new_column)
        else:
            columns.append(column.copy())

    shadow_table_name = db._SHADOW_TABLE_PREFIX + table.name
    shadow_table = Table(shadow_table_name, meta, *columns,
                         mysql_engine='InnoDB')
    try:
        shadow_table.create()
    except (OperationalError, ProgrammingError):
        LOG.info(repr(shadow_table))
        LOG.exception(_('Exception while creating table.'))
        raise exception.ShadowTableExists(name=shadow_table_name)
    except Exception:
        LOG.info(repr(shadow_table))
        LOG.exception(_('Exception while creating table.'))

def _get_default_deleted_value(table):
    if isinstance(table.c.id.type, Integer):
        return 0
    if isinstance(table.c.id.type, String):
        return ""
    raise exception.NovaException(_("Unsupported id columns type"))

def _restore_indexes_on_deleted_columns(migrate_engine, table_name, indexes):
    table = get_table(migrate_engine, table_name)

    insp = reflection.Inspector.from_engine(migrate_engine)
    real_indexes = insp.get_indexes(table_name)
    existing_index_names = dict([(index['name'], index['column_names'])
                                  for index in real_indexes])

    # NOTE(boris-42): Restore indexes on `deleted` column
    for index in indexes:
        if 'deleted' not in index['column_names']:
            continue
        name = index['name']
        if name in existing_index_names:
            column_names = [table.c[c] for c in existing_index_names[name]]
            old_index = Index(name, *column_names, unique=index["unique"])
            old_index.drop(migrate_engine)

        column_names = [table.c[c] for c in index['column_names']]
        new_index = Index(index["name"], *column_names, unique=index["unique"])
        new_index.create(migrate_engine)

def change_deleted_column_type_to_boolean(migrate_engine, table_name,
                                          **col_name_col_instance):
    if migrate_engine.name == "sqlite":
        return _change_deleted_column_type_to_boolean_sqlite(migrate_engine,
                                                    table_name,
                                                    **col_name_col_instance)
    insp = reflection.Inspector.from_engine(migrate_engine)
    indexes = insp.get_indexes(table_name)

    table = get_table(migrate_engine, table_name)

    old_deleted = Column('old_deleted', Boolean, default=False)
    old_deleted.create(table, populate_default=False)

    table.update().\
            where(table.c.deleted == table.c.id).\
            values(old_deleted=True).\
            execute()

    table.c.deleted.drop()
    table.c.old_deleted.alter(name="deleted")

    _restore_indexes_on_deleted_columns(migrate_engine, table_name, indexes)

def _change_deleted_column_type_to_boolean_sqlite(migrate_engine, table_name,
                                                  **col_name_col_instance):
    insp = reflection.Inspector.from_engine(migrate_engine)
    table = get_table(migrate_engine, table_name)

    columns = []
    for column in table.columns:
        column_copy = None
        if column.name != "deleted":
            if isinstance(column.type, NullType):
                column_copy = _get_not_supported_column(col_name_col_instance,
                                                        column.name)
            else:
                column_copy = column.copy()
        else:
            column_copy = Column('deleted', Boolean, default=0)
        columns.append(column_copy)

    constraints = [constraint.copy() for constraint in table.constraints]

    meta = MetaData(bind=migrate_engine)
    new_table = Table(table_name + "__tmp__", meta,
                      *(columns + constraints))
    new_table.create()

    indexes = []
    for index in insp.get_indexes(table_name):
        column_names = [new_table.c[c] for c in index['column_names']]
        indexes.append(Index(index["name"], *column_names,
                             unique=index["unique"]))

    c_select = []
    for c in table.c:
        if c.name != "deleted":
            c_select.append(c)
        else:
            c_select.append(table.c.deleted == table.c.id)

    ins = InsertFromSelect(new_table, select(c_select))
    migrate_engine.execute(ins)

    table.drop()
    [index.create(migrate_engine) for index in indexes]

    new_table.rename(table_name)
    new_table.update().\
        where(new_table.c.deleted == new_table.c.id).\
        values(deleted=True).\
        execute()

def change_deleted_column_type_to_id_type(migrate_engine, table_name,
                                          **col_name_col_instance):
    if migrate_engine.name == "sqlite":
        return _change_deleted_column_type_to_id_type_sqlite(migrate_engine,
                                                    table_name,
                                                    **col_name_col_instance)
    insp = reflection.Inspector.from_engine(migrate_engine)
    indexes = insp.get_indexes(table_name)

    table = get_table(migrate_engine, table_name)

    new_deleted = Column('new_deleted', table.c.id.type,
                         default=_get_default_deleted_value(table))
    new_deleted.create(table, populate_default=True)

    table.update().\
            where(table.c.deleted == True).\
            values(new_deleted=table.c.id).\
            execute()
    table.c.deleted.drop()
    table.c.new_deleted.alter(name="deleted")

    _restore_indexes_on_deleted_columns(migrate_engine, table_name, indexes)

def _change_deleted_column_type_to_id_type_sqlite(migrate_engine, table_name,
                                                  **col_name_col_instance):
    # NOTE(boris-42): sqlaclhemy-migrate can't drop column with check
    #                 constraints in sqlite DB and our `deleted` column has
    #                 2 check constraints. So there is only one way to remove
    #                 these constraints:
    #                 1) Create new table with the same columns, constraints
    #                 and indexes. (except deleted column).
    #                 2) Copy all data from old to new table.
    #                 3) Drop old table.
    #                 4) Rename new table to old table name.
    insp = reflection.Inspector.from_engine(migrate_engine)
    meta = MetaData(bind=migrate_engine)
    table = Table(table_name, meta, autoload=True)
    default_deleted_value = _get_default_deleted_value(table)

    columns = []
    for column in table.columns:
        column_copy = None
        if column.name != "deleted":
            if isinstance(column.type, NullType):
                column_copy = _get_not_supported_column(col_name_col_instance,
                                                        column.name)
            else:
                column_copy = column.copy()
        else:
            column_copy = Column('deleted', table.c.id.type,
                                 default=default_deleted_value)
        columns.append(column_copy)

    def is_deleted_column_constraint(constraint):
        # NOTE(boris-42): There is no other way to check is CheckConstraint
        #                 associated with deleted column.
        if not isinstance(constraint, CheckConstraint):
            return False
        sqltext = str(constraint.sqltext)
        return (sqltext.endswith("deleted in (0, 1)") or
                sqltext.endswith("deleted IN (:deleted_1, :deleted_2)"))

    constraints = []
    for constraint in table.constraints:
        if not is_deleted_column_constraint(constraint):
            constraints.append(constraint.copy())

    new_table = Table(table_name + "__tmp__", meta,
                      *(columns + constraints))
    new_table.create()

    indexes = []
    for index in insp.get_indexes(table_name):
        column_names = [new_table.c[c] for c in index['column_names']]
        indexes.append(Index(index["name"], *column_names,
                             unique=index["unique"]))

    ins = InsertFromSelect(new_table, table.select())
    migrate_engine.execute(ins)

    table.drop()
    [index.create(migrate_engine) for index in indexes]

    new_table.rename(table_name)
    new_table.update().\
        where(new_table.c.deleted == True).\
        values(deleted=new_table.c.id).\
        execute()

    # NOTE(boris-42): Fix value of deleted column: False -> "" or 0.
    new_table.update().\
        where(new_table.c.deleted == False).\
        values(deleted=default_deleted_value).\
        execute()

def _add_index(migrate_engine, table, index_name, idx_columns):
    index = Index(
        index_name, *[getattr(table.c, col) for col in idx_columns]
    )
    index.create()

def _drop_index(migrate_engine, table, index_name, idx_columns):
    index = Index(
        index_name, *[getattr(table.c, col) for col in idx_columns]
    )
    index.drop()

def _change_index_columns(migrate_engine, table, index_name,
                          new_columns, old_columns):
    Index(
        index_name,
        *[getattr(table.c, col) for col in old_columns]
    ).drop(migrate_engine)

    Index(
        index_name,
        *[getattr(table.c, col) for col in new_columns]
    ).create()

def modify_indexes(migrate_engine, data, upgrade=True):
    if migrate_engine.name == 'sqlite':
        return

    meta = MetaData()
    meta.bind = migrate_engine

    for table_name, indexes in data.iteritems():
        table = Table(table_name, meta, autoload=True)

        for index_name, old_columns, new_columns in indexes:
            if not upgrade:
                new_columns, old_columns = old_columns, new_columns

            if migrate_engine.name == 'postgresql':
                if upgrade:
                    _add_index(migrate_engine, table, index_name, new_columns)
                else:
                    _drop_index(migrate_engine, table, index_name, old_columns)
            elif migrate_engine.name == 'mysql':
                _change_index_columns(migrate_engine, table, index_name,
                                      new_columns, old_columns)
            else:
                raise ValueError('Unsupported DB %s' % migrate_engine.name)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright 2012 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Command-line flag library.

Emulates gflags by wrapping cfg.ConfigOpts.

The idea is to move fully to cfg eventually, and this wrapper is a
stepping stone.

"""

import os
import socket
import sys

from oslo.config import cfg
from vsm import version

FLAGS = cfg.CONF

def parse_args(argv, default_config_files=None):
    FLAGS(argv[1:], project='vsm',
          version=version.version_string(),
          default_config_files=default_config_files)

class UnrecognizedFlag(Exception):
    pass

def DECLARE(name, module_string, flag_values=FLAGS):
    if module_string not in sys.modules:
        __import__(module_string, globals(), locals())
    if name not in flag_values:
        raise UnrecognizedFlag('%s not defined by %s' % (name, module_string))

def _get_my_ip():
    """
    Returns the actual ip of the local machine.

    This code figures out what source address would be used if some traffic
    were to be sent out to some well known address on the Internet. In this
    case, a Google DNS server is used, but the specific address does not
    matter much.  No traffic is actually sent.
    """
    try:
        csock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        csock.connect(('8.8.8.8', 80))
        (addr, port) = csock.getsockname()
        csock.close()
        return addr
    except socket.error:
        return "127.0.0.1"

core_opts = [
    cfg.StrOpt('connection_type',
               default=None,
               help='Virtualization api connection type : libvirt, xenapi, '
                    'or fake'),
    cfg.StrOpt('sql_connection',
               default='sqlite:///$state_path/$sqlite_db',
               help='The SQLAlchemy connection string used to connect to the '
                    'database',
               secret=True),
    cfg.IntOpt('sql_connection_debug',
               default=0,
               help='Verbosity of SQL debugging information. 0=None, '
                    '100=Everything'),
    cfg.StrOpt('api_paste_config',
               default="api-paste.ini",
               help='File name for the paste.deploy config for vsm-api'),
    cfg.StrOpt('pybasedir',
               default=os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                    '../')),
               help='Directory where the vsm python module is installed'),
    cfg.StrOpt('bindir',
               default='$pybasedir/bin',
               help='Directory where vsm binaries are installed'),
    cfg.StrOpt('state_path',
               default='$pybasedir',
               help="Top-level directory for maintaining vsm's state"), ]

debug_opts = [
]

tests_opts = [
    cfg.StrOpt('testdb_manager',
               default='vsm.tests.db.manager.TestDBManager',
               help='full class name for the Manager for storage backup'),
    cfg.StrOpt('testdb_topic',
               default='vsm-testdb',
               help='the topic testdb nodes listen on'),
]

FLAGS.register_cli_opts(core_opts)
FLAGS.register_cli_opts(debug_opts)
FLAGS.register_cli_opts(tests_opts)

global_opts = [
    cfg.StrOpt('bypath_dir',
               default='/dev/disk/by-path/',
               help='by-path dir'),
    cfg.StrOpt('cluster_manifest',
               default='/etc/manifest/cluster.manifest',
               help='cluster.manifest'),
    cfg.StrOpt('vsm_config',
               default='/etc/vsm/vsm.conf',
               help='vsm.conf'),
    cfg.StrOpt('crushmap_bin',
               default='/etc/vsm/crushmap.bin',
               help='crushmap.bin'),
    cfg.StrOpt('crushmap_src',
               default='/etc/vsm/crushmap.src',
               help='crushmap.src'),
    cfg.StrOpt('default_weight',
               default=0.1,
               help='default weight of empty item'),
    cfg.StrOpt('ceph_conf_template',
               default='/etc/vsm/ceph.conf.template',
               help='Default template of ceph.conf'),
    cfg.StrOpt('mon_keyring',
               default='/etc/ceph/ceph.mon.keyring',
               help='Key ring localtion.'),
    cfg.StrOpt('vsm_user',
               default='vsm',
               help='user name of vsm'),
    cfg.StrOpt('my_ip',
               default=_get_my_ip(),
               help='ip address of this host'),
    cfg.StrOpt('glance_host',
               default='$my_ip',
               help='default glance hostname or ip'),
    cfg.IntOpt('glance_port',
               default=9292,
               help='default glance port'),
    cfg.ListOpt('glance_api_servers',
                default=['$glance_host:$glance_port'],
                help='A list of the glance api servers available to vsm '
                     '([hostname|ip]:port)'),
    cfg.IntOpt('glance_api_version',
               default=1,
               help='Version of the glance api to use'),
    cfg.IntOpt('glance_num_retries',
               default=0,
               help='Number retries when downloading an image from glance'),
    cfg.BoolOpt('glance_api_insecure',
                default=False,
                help='Allow to perform insecure SSL (https) requests to '
                'glance'),
    cfg.StrOpt('agent_topic',
               default='vsm-agent',
               help='the topic agent nodes listen on'),
    cfg.StrOpt('ml_topic',
               default='vsm-ml',
               help='the topic ML nodes listen on'),
    cfg.StrOpt('conductor_topic',
               default='vsm-conductor',
               help='the topic conductor nodes listen on'),
    cfg.StrOpt('physical_topic',
               default='vsm-physical',
               help='the topic physical nodes listen on'),
    cfg.StrOpt('operation_topic',
               default='vsm-operation',
               help='the topic operation nodes listen on'),
    cfg.StrOpt('scheduler_topic',
               default='vsm-scheduler',
               help='the topic scheduler nodes listen on'),
    cfg.StrOpt('publisher_topic',
               default='vsm-publisher',
               help='the topic publisher nodes listen on'),
    cfg.StrOpt('storage_topic',
               default='vsm-storage',
               help='the topic storage nodes listen on'),
    cfg.StrOpt('backup_topic',
               default='vsm-backup',
               help='the topic storage backup nodes listen on'),
    cfg.BoolOpt('enable_v1_api',
                default=True,
                help=_("Deploy v1 of the Vsm API. ")),
    cfg.BoolOpt('enable_v2_api',
                default=True,
                help=_("Deploy v2 of the Vsm API. ")),
    cfg.BoolOpt('api_rate_limit',
                default=True,
                help='whether to rate limit the api'),
    cfg.ListOpt('vsmapi_storage_ext_list',
                default=[],
                help='Specify list of extensions to load when using osapi_'
                     'storage_extension option with vsm.api.contrib.'
                     'select_extensions'),
    cfg.MultiStrOpt('vsmapi_storage_extension',
                    default=['vsm.api.contrib.standard_extensions'],
                    help='osapi storage extension to load'),
    cfg.StrOpt('vsmapi_storage_base_URL',
               default=None,
               help='Base URL that will be presented to users in links '
                    'to the OpenStack Hardware API',
               deprecated_name='osapi_compute_link_prefix'),
    cfg.IntOpt('osapi_max_limit',
               default=1000,
               help='the maximum number of items returned in a single '
                    'response from a collection resource'),
    cfg.StrOpt('sqlite_db',
               default='vsm.sqlite',
               help='the filename to use with sqlite'),
    cfg.BoolOpt('sqlite_synchronous',
                default=True,
                help='If passed, use synchronous mode for sqlite'),
    cfg.IntOpt('sql_idle_timeout',
               default=3600,
               help='timeout before idle sql connections are reaped'),
    cfg.IntOpt('sql_max_retries',
               default=10,
               help='maximum db connection retries during startup. '
                    '(setting -1 implies an infinite retry count)'),
    cfg.IntOpt('sql_retry_interval',
               default=10,
               help='interval between retries of opening a sql connection'),
    cfg.StrOpt('backup_manager',
               default='vsm.backup.manager.BackupManager',
               help='full class name for the Manager for storage backup'),
    cfg.StrOpt('conductor_manager',
               default='vsm.conductor.manager.ConductorManager',
               help='full class name for the Manager for Conductor'),
    cfg.StrOpt('physical_manager',
               default='vsm.physical.manager.PhysicalManager',
               help='full class name for the Manager for Physical'),
    cfg.StrOpt('ntp_keys',
               default='/etc/ntp/ntp.keys',
               help='ntp sync time key'),
    cfg.StrOpt('ssh_key_gen_shell',
               default='/usr/bin/key',
               help='ssh key gen shell'),
    cfg.StrOpt('id_rsa_pub',
               default='/root/.ssh/id_rsa.pub',
               help='id_rsa.pub'),
    cfg.StrOpt('key_name',
               default='/root/.ssh/id_rsa',
               help='the name of key'),
    #TODO change /root/ path.
    cfg.StrOpt('ssh_authorized_keys',
               default='/root/.ssh/authorized_keys',
               help='ssh key gen shell'),
    cfg.StrOpt('ssh_config',
               default='/etc/ssh/ssh_config',
               help='ssh config file path in ~/.ssh'),
    cfg.StrOpt('etc_hosts',
               default='/etc/hosts',
               help='host name list'),
    #TODO should we rerange the flags.py file?
    #such as ceph in a group.
    #Manager in a group.
    cfg.StrOpt('ceph_conf',
               default='/etc/ceph/ceph.conf',
               help='ceph config file path.'),
    cfg.StrOpt('keyring_admin',
               default='/etc/ceph/keyring.admin',
               help='key ring file path.'),
    cfg.StrOpt('monitor_data_path',
               default='/var/lib/ceph/mon/',
               help='Monitor data path to store information.'),
    cfg.StrOpt('osd_data_path',
               default='/var/lib/ceph/osd/',
               help='OSD data path to store information.'),
    cfg.StrOpt('agent_manager',
               default='vsm.agent.manager.AgentManager',
               help='full class name for the Manager for Conductor'),
    cfg.StrOpt('sockclient_port',
               default=8002,
               help='default port of sockclient'),
    cfg.StrOpt('server_manifest',
               default='/etc/manifest/server.manifest',
               help='default path of server.manifest'),
    cfg.IntOpt('ceph_connect_timeout',
               default=100,
               help='The default ceph connect timeout value.'),
    #cfg.StrOpt('id_rsa_pub',
    #           default='/root/.ssh/id_rsa.pub',
    #           help='path of id_rsa.pub'),
    cfg.StrOpt('vsm_config_path',
               default='/etc/vsm/',
               help='configure path of vsm to reside vsm.conf api-paste.ini'),
    cfg.StrOpt('sockclient_manager',
               default='vsm.sockclient.manager.SockClientManager',
               help='full class name for the Manager for Sockclient'),
    cfg.StrOpt('scheduler_manager',
               default='vsm.scheduler.manager.SchedulerManager',
               help='full class name for the Manager for Scheduler'),
    cfg.StrOpt('host',
               default=socket.gethostname().split('.')[0],
               help='Name of this node.  This can be an opaque identifier.  '
                    'It is not necessarily a hostname, FQDN, or IP address.'),
    # NOTE(vish): default to nova for compatibility with nova installs
    cfg.StrOpt('storage_availability_zone',
               default='nova',
               help='availability zone of this node'),
    cfg.ListOpt('memcached_servers',
                default=None,
                help='Memcached servers or None for in process cache.'),
    cfg.StrOpt('default_storage_type',
               default=None,
               help='default storage type to use'),
    cfg.StrOpt('storage_usage_audit_period',
               default='month',
               help='time period to generate storage usages for.  '
                    'Time period must be hour, day, month or year'),
    cfg.StrOpt('root_helper',
               default='sudo',
               help='Deprecated: command to use for running commands as root'),
    cfg.StrOpt('rootwrap_config',
               default=None,
               help='Path to the rootwrap configuration file to use for '
                    'running commands as root'),
    cfg.BoolOpt('monkey_patch',
                default=False,
                help='Whether to log monkey patching'),
    cfg.ListOpt('monkey_patch_modules',
                default=[],
                help='List of modules/decorators to monkey patch'),
    cfg.IntOpt('service_down_time',
               default=60,
               help='maximum time since last check-in for up service'),
    cfg.IntOpt('server_ping_time',
               default=5,
               help='The interval of time to ping other servers.'),
    cfg.IntOpt('ping_count',
               default=5,
               help='The hop that ping will walk.'),
    cfg.IntOpt('time_out',
               default=400,
               help='The default timeout value.'),
    cfg.IntOpt('update_time_interval',
               default=1,
               help='The interval that each server update its update_at field.'
                    'This value should be three times less than other update'
                    'interval, such as server_ping_time'),
    cfg.StrOpt('storage_api_class',
               default='vsm.storage.api.API',
               help='The full class name of the storage API class to use'),
    cfg.StrOpt('backup_api_class',
               default='vsm.backup.api.API',
               help='The full class name of the storage backup API class'),
    cfg.StrOpt('auth_strategy',
               default='noauth',
               help='The strategy to use for auth. Supports noauth, keystone, '
                    'and deprecated.'),
    cfg.ListOpt('enabled_backends',
                default=None,
                help='A list of backend names to use. These backend names '
                     'should be backed by a unique [CONFIG] group '
                     'with its options'),
    cfg.BoolOpt('no_snapshot_gb_quota',
                default=False,
                help='Whether snapshots count against GigaByte quota'),
    cfg.StrOpt('default_zone',
                default='zone_one',
                help='The default one zone name'), ]

FLAGS.register_opts(global_opts)

db_opts = [
    cfg.BoolOpt('enable_new_services',
                default=True,
                help='Services to be added to the available pool on create'),
]

FLAGS.register_opts(db_opts)

disk_worker_opts = [
    cfg.StrOpt('partition_status_missing',
             default='MISSING',
             help='the partition status is missing'),
    cfg.StrOpt('partition_status_ok',
             default='OK',
             help='the partition status is ok'),
]

FLAGS.register_opts(disk_worker_opts)

osd_opts = [
    cfg.StrOpt('osd_state_autoout',
              default='autoout',
              help='the real osd autoout state'),
    cfg.StrOpt('osd_in_up',
              default='In-Up',
              help='osd in and up status'),
    cfg.StrOpt('osd_in_down',
              default='In-Down',
              help='osd in and down status'),
    cfg.StrOpt('osd_out_up',
              default='Out-Up',
              help='osd out and up status'),
    cfg.StrOpt('osd_out_down',
              default='Out-Down',
              help='osd out and down status'),
    cfg.StrOpt('osd_out_down_autoout',
              default='Out-Down-Autoout',
              help='osd out down autoout status'),
]

FLAGS.register_opts(osd_opts)

storage_group_opts = [
    cfg.StrOpt('storage_group_in',
                default='IN',
                help='IN means the storage group is in Ceph'),
    cfg.StrOpt('storage_group_out',
                default='OUT',
                help='OUT means the storage group is not in Ceph'),

]

FLAGS.register_opts(storage_group_opts)

vsm_status_opts = [
    cfg.StrOpt('vsm_status_removed',
              default='Removed',
              help='removed status'),
    cfg.StrOpt('vsm_status_present',
              default='Present',
              help='prisent status'),
    cfg.StrOpt('vsm_status_uninitialized',
          default='Uninitialized',
          help='Uninitialized status'),
]

FLAGS.register_opts(vsm_status_opts)

summary_type_opts = [
    cfg.StrOpt('summary_type_osd',
              default='osd',
              help='summary type osd'),
    cfg.StrOpt('summary_type_pg',
              default='pg',
              help='summary type pg'),
    cfg.StrOpt('summary_type_mon',
              default='mon',
              help='summary type mon'),
    cfg.StrOpt('summary_type_mds',
              default='mds',
              help='summary type mds'),
    cfg.StrOpt('summary_type_cluster',
              default='cluster',
              help='summary type cluster'),
    cfg.StrOpt('summary_type_vsm',
              default='vsm',
              help='summary type vsm'),
    cfg.StrOpt('summary_type_ceph',
              default='ceph',
              help='summary type ceph'),

]

FLAGS.register_opts(summary_type_opts)

vsm_settings_opts = [
    cfg.IntOpt('storage_group_near_full_threshold',
               default=65,
               help='storage group near full threshold'),
    cfg.IntOpt('storage_group_full_threshold',
               default=85,
               help='storage group full threshold'),
    cfg.IntOpt('ceph_status',
               default=60,
               help='ceph status (secs)'),
    cfg.IntOpt('ceph_osd_pool_stats',
               default=60,
               help='ceph osd pool stats (secs)'),
    cfg.IntOpt('reset_pg_heart_beat',
               default=600,
               help='reset pg_num heart beat(secs)'),
    cfg.IntOpt('update_ceph_version_info',
               default=600,
               help='update ceph version info(secs)'),
    cfg.IntOpt('ceph_osd_dump',
               default=60,
               help='ceph osd dump (secs)'),
    cfg.IntOpt('ceph_pg_dump_osds',
               default=600,
               help='ceph pg dump osds (secs)'),
    cfg.IntOpt('ceph_osd_tree',
               default=600,
               help='ceph osd tree (secs)'),
    cfg.IntOpt('ceph_pg_dump_pgs_brief',
               default=1800,
               help='ceph pg dump pgs brief (secs)'),
    cfg.IntOpt('ceph_ec_profile_update',
               default=600,
               help='ceph osd erasure-code-profile get (secs)'),
    cfg.IntOpt('rbd_ls_-l_{pool_name}',
               default=1800,
               help='rbd ls -l {pool_name} (secs)'),
    cfg.IntOpt('ceph_mds_dump',
               default=60,
               help='ceph mds dump (secs)'),
    cfg.IntOpt('cpu_diamond_collect_interval',
               default=0,
               help='cpu data collect interval (secs) with diamond'),
    cfg.IntOpt('ceph_diamond_collect_interval',
               default=0,
               help='ceph perf data collect interval (secs) with diamond'),
    cfg.IntOpt('keep_performance_data_days',
           default=7,
           help='keep performance data days(days)'),
]

FLAGS.register_opts(vsm_settings_opts)




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Common Auth Middleware.

"""
import os

from oslo.config import cfg
import webob.dec
import webob.exc

from vsm.api.openstack import wsgi
from vsm import context
from vsm import flags
from vsm.openstack.common import log as logging
from vsm import wsgi as base_wsgi

use_forwarded_for_opt = cfg.BoolOpt(
    'use_forwarded_for',
    default=False,
    help='Treat X-Forwarded-For as the canonical remote address. '
         'Only enable this if you have a sanitizing proxy.')

FLAGS = flags.FLAGS
FLAGS.register_opt(use_forwarded_for_opt)
LOG = logging.getLogger(__name__)

def pipeline_factory(loader, global_conf, **local_conf):
    """A paste pipeline replica that keys off of auth_strategy."""
    pipeline = local_conf[FLAGS.auth_strategy]
    if not FLAGS.api_rate_limit:
        limit_name = FLAGS.auth_strategy + '_nolimit'
        pipeline = local_conf.get(limit_name, pipeline)
    pipeline = pipeline.split()
    filters = [loader.get_filter(n) for n in pipeline[:-1]]
    app = loader.get_app(pipeline[-1])
    filters.reverse()
    for filter in filters:
        app = filter(app)
    return app

class InjectContext(base_wsgi.Middleware):
    """Add a 'vsm.context' to WSGI environ."""

    def __init__(self, context, *args, **kwargs):
        self.context = context
        super(InjectContext, self).__init__(*args, **kwargs)

    @webob.dec.wsgify(RequestClass=base_wsgi.Request)
    def __call__(self, req):
        req.environ['vsm.context'] = self.context
        return self.application

class VsmKeystoneContext(base_wsgi.Middleware):
    """Make a request context from keystone headers"""

    @webob.dec.wsgify(RequestClass=base_wsgi.Request)
    def __call__(self, req):
        user_id = req.headers.get('X_USER')
        user_id = req.headers.get('X_USER_ID', user_id)
        if user_id is None:
            LOG.debug("Neither X_USER_ID nor X_USER found in request")
            return webob.exc.HTTPUnauthorized()
        # get the roles
        roles = [r.strip() for r in req.headers.get('X_ROLE', '').split(',')]
        if 'X_TENANT_ID' in req.headers:
            # This is the new header since Keystone went to ID/Name
            project_id = req.headers['X_TENANT_ID']
        else:
            # This is for legacy compatibility
            project_id = req.headers['X_TENANT']

        # Get the auth token
        auth_token = req.headers.get('X_AUTH_TOKEN',
                                     req.headers.get('X_STORAGE_TOKEN'))

        # Build a context, including the auth_token...
        remote_address = req.remote_addr
        if FLAGS.use_forwarded_for:
            remote_address = req.headers.get('X-Forwarded-For', remote_address)
        ctx = context.RequestContext(user_id,
                                     project_id,
                                     roles=roles,
                                     auth_token=auth_token,
                                     remote_address=remote_address)

        req.environ['vsm.context'] = ctx
        return self.application

class NoAuthMiddleware(base_wsgi.Middleware):
    """Return a fake token if one isn't specified."""

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        if 'X-Auth-Token' not in req.headers:
            user_id = req.headers.get('X-Auth-User', 'admin')
            project_id = req.headers.get('X-Auth-Project-Id', 'admin')
            os_url = os.path.join(req.url, project_id)
            res = webob.Response()
            # NOTE(vish): This is expecting and returning Auth(1.1), whereas
            #             keystone uses 2.0 auth.  We should probably allow
            #             2.0 auth here as well.
            res.headers['X-Auth-Token'] = '%s:%s' % (user_id, project_id)
            res.headers['X-Server-Management-Url'] = os_url
            res.content_type = 'text/plain'
            res.status = '204'
            return res

        token = req.headers['X-Auth-Token']
        user_id, _sep, project_id = token.partition(':')
        project_id = project_id or user_id
        remote_address = getattr(req, 'remote_address', '127.0.0.1')
        if FLAGS.use_forwarded_for:
            remote_address = req.headers.get('X-Forwarded-For', remote_address)
        ctx = context.RequestContext(user_id,
                                     project_id,
                                     is_admin=True,
                                     remote_address=remote_address)

        req.environ['vsm.context'] = ctx
        return self.application



#    Copyright 2014 Intel
#    All rights reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Client side of the agent RPC API."""
import logging
from oslo.config import cfg

from vsm.openstack.common import jsonutils
from vsm.openstack.common import rpc
import vsm.openstack.common.rpc.proxy

CONF = cfg.CONF

LOG = logging.getLogger(__name__)

class AgentAPI(vsm.openstack.common.rpc.proxy.RpcProxy):
    """Client side of the agent RPC API"""

    BASE_RPC_API_VERSION = '1.0'

    def __init__(self, topic=None):
        super(AgentAPI, self).__init__(
            topic = topic or CONF.agent_topic,
            default_version=self.BASE_RPC_API_VERSION)

    def ping(self, context, arg, timeout=None):
        arg_p = jsonutils.to_primitive(arg)
        msg = self.make_msg('ping', arg=arg_p)
        return self.call(context, msg, version='1.0', timeout=timeout)

    def test_service(self, ctxt, topic, host=None):
        if host:
            topic = rpc.queue_get_for(ctxt, self.topic, host)
        ret = self.call(ctxt, self.make_msg('test_service'), topic, timeout=30, need_try=False)
        return ret

    def update_pool_info(self, ctxt, body=None):
        res = self.cast(ctxt, self.make_msg('update_pool_info', body=body))

    def update_recipe_info(self, ctxt, body=None):
        res = self.call(ctxt, self.make_msg('update_recipe_info', body=body))
        return res

    def present_storage_pools(self, context, body=None):
        return self.cast(context, self.make_msg('present_storage_pools', body=body))

    def revoke_storage_pool(self, context, id):
        return self.call(context, self.make_msg('revoke_storage_pool', id=id))

    def update_keyring_admin_from_db(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                    self.make_msg('update_keyring_admin_from_db'),
                    topic)

    def upload_keyring_admin_into_db(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                    self.make_msg('upload_keyring_admin_into_db'),
                topic, version='1.0', timeout=6000)

    def init_ceph(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context,
                         self.make_msg('init_ceph',
                                       body=body),
                         topic,
                         version='1.0', timeout=6000)

    def add_osd(self, context, host_id, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context,
                         self.make_msg('add_osd',
                                       host_id=host_id),
                         topic,
                         version='1.0', timeout=6000)

    def add_monitor(self, context, host_id, mon_id, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context,
                         self.make_msg('add_monitor',
                                       mon_id=mon_id,
                                       host_id=host_id),
                         topic,
                         version='1.0', timeout=6000)

    def remove_osd(self, context, host_id, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context,
                         self.make_msg('remove_osd',
                                       host_id=host_id),
                         topic,
                         version='1.0', timeout=6000)

    def remove_monitor(self, context, host_id, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context,
                         self.make_msg('remove_monitor',
                                       host_id=host_id),
                         topic,
                         version='1.0', timeout=6000)

    def remove_mds(self, context, host_id, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context,
                         self.make_msg('remove_mds',
                                       host_id=host_id),
                         topic,
                         version='1.0', timeout=6000)

    def add_mds(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context,
                         self.make_msg('add_mds'),
                         topic,
                         version='1.0', timeout=6000)

    def get_ceph_disk_list(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context,
                        self.make_msg('get_ceph_disk_list',),
                        topic, version='1.0', timeout=6000)

    def get_ceph_config(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context,
                        self.make_msg('get_ceph_config',),
                        topic, version='1.0', timeout=6000)

    def save_ceph_config(self, context, config, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                        self.make_msg('save_ceph_config', config=config),
                    topic, version='1.0', timeout=6000)

    def get_ceph_admin_keyring(self, context, host,):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                        self.make_msg('get_ceph_admin_keyring',),
                    topic, version='1.0', timeout=6000)

    def save_ceph_admin_keyring(self, context, keyring_str, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                         self.make_msg('save_ceph_admin_keyring',
                                      keyring_str=keyring_str),
                     topic, version='1.0', timeout=6000)

    def clean_ceph_data(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                         self.make_msg('clean_ceph_data'),
                         topic,
                         version='1.0', timeout=6000)

    def mount_disks(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                         self.make_msg('mount_disks'),
                         topic,
                         version='1.0', timeout=6000)

    def start_osd_daemon(self, context, number, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                         self.make_msg('start_osd_daemon',
                                       num=number),
                         topic,
                         version='1.0', timeout=6000)

    def get_ceph_health(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context,
                         self.make_msg('get_ceph_health'),
                         topic, version='1.0', timeout=6000)

    def get_ceph_health_list(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                         self.make_msg('get_ceph_health_list'),
                         topic, version='1.0', timeout=6000)

    def get_osds_total_num(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context, self.make_msg('get_osds_total_num'), topic,
                version='1.0', timeout=6000)
 
    def start_ceph(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        return self.call(context, self.make_msg('start_ceph'), topic,
                version='1.0', timeout=6000)

    def create_storage_pool(self, ctxt, body=None):
        return self.call(ctxt, self.make_msg('create_storage_pool', body=body))

    def get_pool_id_by_name(self, context, name, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.call(context,
                      self.make_msg('get_pool_id_by_name',
                                    name=name),
                      topic, version='1.0', timeout=6000)
        return res

    def set_crushmap(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.call(context,
                        self.make_msg('set_crushmap'),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def update_ssh_keys(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.call(context,
                        self.make_msg('update_ssh_keys'),
                        topic, version='1.0', timeout=6000)
        return res
    def get_smart_info(self, context, host, device):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.call(context,
                        self.make_msg('get_smart_info', device=device),
                        topic, version='1.0', timeout=6000)
        return res
    def create_crushmap(self, context, server_list, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.call(context,
                        self.make_msg('create_crushmap',
                                      server_list=server_list),
                        topic, version='1.0', timeout=6000)
        return res

    def refresh_osd_num(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.cast(context,
                        self.make_msg('refresh_osd_number'),
                        topic, version='1.0', timeout=6000)
        return res

    def add_new_zone(self, context, zone_name, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.call(context,
                        self.make_msg('add_new_zone',
                                        zone_name=zone_name),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def start_server(self, context, node_id, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.call(context,
                        self.make_msg('start_server',
                                       node_id=node_id),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def stop_server(self, context, node_id, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.call(context,
                        self.make_msg('stop_server',
                                       node_id=node_id),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def ceph_upgrade(self, context, node_id, host, key_url, pkg_url,restart):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.call(context,
                        self.make_msg('ceph_upgrade',node_id=node_id,key_url=key_url,pkg_url=pkg_url,restart=restart),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def osd_remove(self, context, osd_id, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        #self.test_service(context, topic)
        res = self.call(context,
                      self.make_msg('osd_remove',
                                    osd_id=osd_id),
                      topic, version='1.0', timeout=6000)
        return res

    def osd_restart(self, context, osd_id, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        #self.test_service(context, topic)
        res = self.call(context,
                      self.make_msg('osd_restart',
                                    osd_id=osd_id),
                      topic, version='1.0', timeout=6000)
        return res

    def osd_add(self, context, osd_id, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        #self.test_service(context, topic)
        res = self.call(context,
                      self.make_msg('osd_add',
                                    osd_id=osd_id),
                      topic, version='1.0', timeout=6000)
        return res

    def osd_restore(self, context, osd_id, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        #self.test_service(context, topic)
        res = self.call(context,
                      self.make_msg('osd_restore',
                                    osd_id=osd_id),
                      topic, version='1.0', timeout=6000)
        return res

    def osd_refresh(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('osd_refresh'),
                        topic, version='1.0', timeout=6000)
        return res

    def cluster_refresh(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('cluster_refresh'),
                        topic, version='1.0', timeout=6000)
        return res

    def integrate_cluster_update_status(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('integrate_cluster_update_status'),
                        topic, version='1.0', timeout=6000)
        return res

    def integrate_cluster_sync_osd_states(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('integrate_cluster_sync_osd_states'),
                        topic, version='1.0', timeout=6000)
        return res

    def integrate_cluster_from_ceph(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('integrate_cluster_from_ceph'),
                        topic, version='1.0', timeout=6000)
        return res

    def cluster_id(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                      self.make_msg('cluster_id'),
                  topic, version='1.0', timeout=6000)
        return res

    def update_osd_state(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.cast(context, self.make_msg('update_osd_state'), topic)

    def update_pool_state(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context, self.make_msg('update_pool_state'), topic)

    def update_mon_state(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.cast(context, self.make_msg('update_mon_health'), topic)

    def set_pool_pg_pgp_num(self, context, host, pool, pg_num, pgp_num):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.cast(context, self.make_msg('set_pool_pg_pgp_num',
                  pool=pool, pg_num=pg_num, pgp_num=pgp_num), topic)

    def update_all_status(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.cast(context, self.make_msg('update_all_status'), topic)

    def update_ceph_conf(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.cast(context, self.make_msg('update_ceph_conf'), topic)


    def start_monitor(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context, self.make_msg('start_monitor'), topic,
                        version='1.0', timeout=6000)

    def start_mds(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context, self.make_msg('start_mds'), topic,
                        version='1.0', timeout=6000)

    def start_osd(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context, self.make_msg('start_osd'), topic,
                        version='1.0', timeout=6000)

    def inital_ceph_osd_db_conf(self, context, server_list,ceph_conf_in_cluster_manifest,host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                         self.make_msg('inital_ceph_osd_db_conf',
                                       server_list=server_list,
                                       ceph_conf_in_cluster_manifest=ceph_conf_in_cluster_manifest),
                         topic,
                         version='1.0',
                         timeout=6000)

    def mkcephfs(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                         self.make_msg('mkcephfs'),
                         topic,
                         version='1.0',
                         timeout=6000)

    def stop_mds(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context, self.make_msg('stop_mds'), topic,
                        version='1.0', timeout=6000)

    def health_status(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context, self.make_msg('health_status'), topic)

    def write_monitor_keyring(self, context, monitor_keyring, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        return self.call(context,
                         self.make_msg('write_monitor_keyring',
                                        monitor_keyring=monitor_keyring),
                         topic,
                         version='1.0', timeout=6000)

    def track_monitors(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('track_monitors'),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def create_keyring(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('create_keyring'),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def prepare_osds(self, context, server_list, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('prepare_osds',
                        server_list=server_list),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def add_cache_tier(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('add_cache_tier',
                                      body=body),
                        topic,
                        version='1.0', timeout=6000)

    def remove_cache_tier(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('remove_cache_tier',
                                      body=body),
                        topic,
                        version='1.0', timeout=6000)

    def start_cluster(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.call(context,
                        self.make_msg('start_cluster'),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def stop_cluster(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        self.test_service(context, topic)
        res = self.call(context,
                        self.make_msg('stop_cluster'),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def monitor_restart(self, context, monitor_num, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        #self.test_service(context, topic)
        res = self.call(context,
                      self.make_msg('monitor_restart',
                                    monitor_num=monitor_num),
                      topic, version='1.0', timeout=6000)
        return res

    def get_available_disks(self, context, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('get_available_disks'),
                        topic, version='1.0', timeout=6000)
        return res

    def add_new_disks_to_cluster(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('add_new_disks_to_cluster',
                                      body=body),
                        topic,
                        version='1.0', timeout=6000)

    def reconfig_diamond(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.cast(context,
                        self.make_msg('reconfig_diamond',
                                      body=body),
                        topic)

    def check_pre_existing_cluster(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('check_pre_existing_cluster',
                                      body=body),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def import_cluster(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('import_cluster',
                                      body=body),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def detect_cephconf(self, context, keyring, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('detect_cephconf',
                                      keyring=keyring),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def detect_crushmap(self, context, keyring, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('detect_crushmap',
                                      keyring=keyring),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def add_rule_to_crushmap(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('add_rule_to_crushmap',
                                      body=body),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def modify_rule_in_crushmap(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('modify_rule_in_crushmap',
                                      body=body),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def update_zones_from_crushmap_to_db(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('update_zones_from_crushmap_to_db',
                                      body=body),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def update_storage_groups_from_crushmap_to_db(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('update_storage_groups_from_crushmap_to_db',
                                      body=body),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def add_zone_to_crushmap_and_db(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('add_zone_to_crushmap_and_db',
                                      body=body),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def get_default_pg_num_by_storage_group(self, context, body, host):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context,
                        self.make_msg('get_default_pg_num_by_storage_group',
                                      body=body),
                        topic,
                        version='1.0', timeout=6000)
        return res

    def rgw_create(self, context, name, host, keyring, log_file, rgw_frontends,
                   is_ssl, s3_user_uid, s3_user_display_name, s3_user_email,
                   swift_user_subuser, swift_user_access, swift_user_key_type):
        topic = rpc.queue_get_for(context, self.topic, host)
        res = self.call(context, self.make_msg('rgw_create',
                                               name=name,
                                               host=host,
                                               keyring=keyring,
                                               log_file=log_file,
                                               rgw_frontends=rgw_frontends,
                                               is_ssl=is_ssl,
                                               s3_user_uid=s3_user_uid,
                                               s3_user_display_name=s3_user_display_name,
                                               s3_user_email=s3_user_email,
                                               swift_user_subuser=swift_user_subuser,
                                               swift_user_access=swift_user_access,
                                               swift_user_key_type=swift_user_key_type),
                        topic, version='1.0', timeout=6000)
        return res


# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common
import logging
import time

LOG = logging.getLogger(__name__)

class ViewBuilder(common.ViewBuilder):
    _collection_name = "mdses"

    def basic(self, request, mds):
        LOG.info("mdses api view %s " % mds)
        _mds = {
            "mds": {
                "id": mds['id'],
                "name": mds['name'],
                "gid": mds['gid'],
                "state": mds['state'],
                "address": mds['address']
            }
        }
        _mds['mds']['updated_at'] = time.strftime("%Y-%m-%d %H:%M:%S", time.strptime(mds['updated_at'], "%Y-%m-%dT%H:%M:%S.000000"))
        return _mds

    def _detail(self, request, mds):
        LOG.info("mdses api detail view %s " % mds)
        _mds = {
            "mds": {
                "id": mds['id'],
                "name": mds['name'],
                "gid": mds['gid'],
                "state": mds['state'],
                "address": mds['address']
            }
        }
        try:
            _mds['mds']['updated_at'] = time.strftime("%Y-%m-%d %H:%M:%S", time.strptime(mds['updated_at'], "%Y-%m-%dT%H:%M:%S.000000"))
        except:
            _mds['mds']['updated_at'] = ""
        return _mds

    def detail(self, request, mdses):
        return self._list_view(self._detail, request, mdses)

    def index(self, request, mdses):
        """Show a list of mdses without many details."""
        return self._list_view(self.basic, request, mdses)

    def _list_view(self, func, request, mdses):
        """Provide a view for a list of mdses."""
        mds_list = [func(request, mds)["mds"] for mds in mdses]
        mdses_dict = dict(mdses=mds_list)
        return mdses_dict

